// Sample program for use with Sangfrapi.sys
// Copyright (c) 1996 Sangoma Technologies Inc.
// Use at your own risk.
// Target: Win32 Console

#include <windows.h>
#include <winioctl.h>
#include <stdio.h>
#include <conio.h>
#include "sanfr8in.h"

PFRAME_RELAY_CBLOCK pcb;
UCHAR rxdata[4000];
UCHAR cbdata[4000];

#define BUILD_S514

#ifdef BUILD_S514
	char *device_name = "\\\\.\\SANFR514C0";
#else
	char *device_name = "\\\\.\\SANFR8C0";
#endif

char *hellostring = "LET ME GO! I'VE BEEN FRAMED!";

UCHAR DoCommand(HANDLE drv, PFRAME_RELAY_CBLOCK pcb);
UCHAR DoWriteCommand(HANDLE drv, PFRAME_RELAY_CBLOCK pcb);
UCHAR DoReadCommand(HANDLE drv, PUCHAR dat, ULONG maxl);
DWORD ThreadFunc(LPDWORD lpdwParam);
UCHAR DoStatusCommand(HANDLE drv, PFRAME_RELAY_STATUS_INFO frsi);

main()
{
	DWORD ln;
	PFRAME_RELAY_CONFIG frc;
	FRAME_RELAY_STATUS_INFO statinfo;
    DWORD dwThreadId, dwThrdParam = 1; 
    HANDLE hThread; 
	HANDLE drv;
	char c;
	UCHAR Station;
	UCHAR Clocking;

	pcb = (PFRAME_RELAY_CBLOCK)cbdata;

	// Open device number 0
	drv = CreateFile(device_name, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, (LPSECURITY_ATTRIBUTES)NULL,
			OPEN_EXISTING, FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH, (HANDLE)NULL);
    if (drv == INVALID_HANDLE_VALUE) {
		printf("Unable to open %s!\n", device_name);
		return(0);
	}

	// clear the configuration data area
	memset(cbdata, 0, sizeof(cbdata));

	// set up for a global frame relay configuration
	frc = (PFRAME_RELAY_CONFIG)pcb->Data;
	pcb->Command = SET_DLCI_CONFIGURATION;
	pcb->BufferLength = sizeof(FRAME_RELAY_CONFIG);
	pcb->DLCI = 0;

	// in this case we are configuring an Access Node
	Station = 1;
	frc->Station = 1;
	printf("\nConfigure for 'c'pe or 's'witch? ");
	if (getche() == 'c') {
		frc->Station = 0;
		Station = 0;
	}
	printf("\n");
	frc->ConfigBits = 0x2001;
	if (Station == 0)
		frc->ConfigBits = 0xa001;	// autoconfigure dlcis
	frc->BaudRateKbps = 56;			// 56 kbps
	Clocking = 0x00;
	printf("Configure for 'i'nternal or 'e'xternal clocking? ");
	if (getche() == 'i')
		Clocking |= 0x01;
	printf("\nConfigure for 'r's-232 or 'v'.35? ");
	if (getche() == 'r')
		Clocking |= 0x02;
	printf("\n");
	frc->ClockingInterface = Clocking;
	frc->MaxIFrameLength = 1500;
	frc->T391 = 10;
	frc->T392 = 15;
	frc->N391 = 6;
	frc->N392 = 3;
	frc->N393 = 4;
	frc->CIRfwd = 16;
	frc->Bcfwd = 16;
	frc->CIRbwd = 16;
	frc->Bcbwd = 16;
	frc->DLCI[0] = 16;

	// perform the command
	DoCommand(drv, pcb);

	// display the return code - this command returns the actual bit rate
	printf("Rcode = %02X, blength = %u\n", pcb->ReturnCode, pcb->BufferLength);
	for (ln=0; ln < (DWORD)pcb->BufferLength; ln++)
		printf("%02X ", pcb->Data[ln]);
	printf("\n");

	// enable the transmitter and receiver
	pcb->Command = ENABLE_COMMUNICATIONS;
	pcb->BufferLength = 0;
	DoCommand(drv, pcb);

	if (Station) {
		// add dlci number 16
		pcb->Command = ADD_DLCI;
		pcb->BufferLength = 2;
		*(USHORT *)&pcb->Data[0] = 16;
		DoCommand(drv, pcb);

		// activate dlci number 16
		pcb->Command = ACTIVATE_DLCI;
		pcb->BufferLength = 2;
		*(USHORT *)&pcb->Data[0] = 16;
		DoCommand(drv, pcb);
	}

	// start a read thread
    hThread = CreateThread( 
        NULL,                        /* no security attributes        */ 
        0,                           /* use default stack size        */ 
        (LPTHREAD_START_ROUTINE) ThreadFunc, /* thread function       */ 
        &dwThrdParam,                /* argument to thread function   */ 
        0,                           /* use default creation flags    */ 
        &dwThreadId);                /* returns the thread identifier */ 

	

	// loop around waiting for user to request the transmission of a frame
	// or to look at some status bytes
	do {
		printf("\nPress any key to send hello or 'q' to quit or 's' for status...");
		c=getch();
		printf("\n");
		if (c == 's') {
			// print status information
			DoStatusCommand(drv, &statinfo);
			printf("Base port: %02X\nException Byte: %02X\nModem Byte: %02X\n",
				statinfo.BasePortRegister, statinfo.ExceptionConditionByte, statinfo.ModemStatusByte);
		} else if (c != 'q') {
			// any other key besides 'q'uit sends a frame
			pcb->DLCI = 16;				// send on dlci number 16
			pcb->FecnBecnDeCr = 0;		// don't set any flags
			pcb->BufferLength = strlen(hellostring);
			strcpy(pcb->Data, hellostring);
			printf("Doing write right now...\n");
			// queue for transmission
			DoWriteCommand(drv, pcb);
			// if the return code isn't zero, it didn't get queued
			printf("Write rcode = %02X.\n", pcb->ReturnCode);
		}
	} while (c != 'q');

	printf("\nPress any key to disable comms...");
	getch();

	// turn off the receiver - this also cancels any pending read commands
	do {
		pcb->Command = DISABLE_COMMUNICATIONS;
		pcb->BufferLength = 0;
		DoCommand(drv, pcb);
		printf("Disable comms rcode = %02X.\n", pcb->ReturnCode);
	} while (pcb->ReturnCode);

	printf("Press any key to close device...\n");
	getch();

	// close the open handle
	CloseHandle(drv);
	return(0);
}

// perform a standard frame relay command
UCHAR DoCommand(HANDLE drv, PFRAME_RELAY_CBLOCK pcb)
{
	DWORD ln;

	if (DeviceIoControl(
		drv,
		IoctlCodeCblockCommand,
		(LPVOID)pcb,
		(ULONG)pcb->BufferLength + 16L,
		(LPVOID)pcb,
		sizeof(cbdata),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		) == FALSE)
		pcb->ReturnCode = 0x7d;		// STATUS_UNSUCCESSFUL
	return(pcb->ReturnCode);
}

// transmit a frame
UCHAR DoWriteCommand(HANDLE drv, PFRAME_RELAY_CBLOCK pcb)
{
	DWORD ln;

	if (DeviceIoControl(
		drv,
		IoctlCodeWriteCommand,
		(LPVOID)pcb,
		(ULONG)pcb->BufferLength + 16L,
		(LPVOID)pcb,
		sizeof(cbdata),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		) == FALSE)
		pcb->ReturnCode = 0x7d;		// STATUS_UNSUCCESSFUL
	return(pcb->ReturnCode);
}

// do blocking read command
// received frame is stored in user buffer on return
UCHAR DoReadCommand(HANDLE drv, PUCHAR dat, ULONG maxl)
{
	DWORD ln;

	*dat = 0;				// initialize Status member to 0x00
	if (DeviceIoControl(
		drv,
		IoctlCodeReadCommand,
		(LPVOID)NULL,
		0L,
		(LPVOID)dat,
		maxl,
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		) == FALSE)
		*dat = 0x7d;		// STATUS_UNSUCCESSFUL
	return(*dat);			// return value of Status member
}

// read some status information off the board
UCHAR DoStatusCommand(HANDLE drv, PFRAME_RELAY_STATUS_INFO frsi)
{
	DWORD ln;

	if (DeviceIoControl(
		drv,
		IoctlCodeStatusCommand,
		NULL,
		0,
		(LPVOID)frsi,
		sizeof(FRAME_RELAY_STATUS_INFO),
		(LPDWORD)(&ln),
		(LPOVERLAPPED)NULL
		) == FALSE) return(0);	// STATUS_UNSUCCESSFUL
		
	return(1);
}

// read thread
DWORD ThreadFunc(LPDWORD lpdwParam)
{ 
	USHORT datlen;
	DWORD ln;
	HANDLE drv;
	PFRAME_RELAY_READ_INFO pri;

    printf("\nThreadFunc:  thread parameter=%d\n", *lpdwParam); 

	// open another handle strictly for reading
	drv = CreateFile(device_name, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, (LPSECURITY_ATTRIBUTES)NULL,
			OPEN_EXISTING, FILE_FLAG_NO_BUFFERING | FILE_FLAG_WRITE_THROUGH, (HANDLE)NULL);
    if (drv == INVALID_HANDLE_VALUE) {
		printf("\nUnable to open %s!\n", device_name);
		return(0);
	}

	// keep reading forever until the attempt fails
	do {
		printf("\nWaiting to read something...\n");
		// use our special buffer at rxdata to hold received frame
		printf("\ndrc returns %02X...\n", DoReadCommand(drv, rxdata, 4000));
		pri = (PFRAME_RELAY_READ_INFO)rxdata;
		if (pri->Status == 0x01) {
			// a frame was received - dump it along with first 16 header bytes
			datlen = pri->FrameLength;
			for (ln=0; ln < (DWORD)datlen+16; ln++)
				printf("%02X ", rxdata[ln]);
			printf("\n");
		}
	} while (pri->Status == 0x01);

	// if the read command returned but with no frame, either
	// DISABLE_COMMUNICATIONS was done or all open device handles were closed
	printf("\nThread is closing the driver...\n");
	CloseHandle(drv);
	printf("Thread is terminating...\n");
    return 0; 
} 
